{ Functions calls derived from the fossil revision 5 documentation }
{ and will work with any proper fossil driver (X00, BNU, OPUSCOMM) }

unit Fossil;

interface

uses DOS;

const
  MaxPort = 4;

var
  FossilInstalled : boolean;
  FossilMaxFcn    : byte;
  FossilVersion   : byte;
  PortStat        : array [0..MaxPort] of word;
  PortUsed        : integer;
  FossilExitSave  : pointer;

procedure InitFossil    (port : integer);
procedure XmitPort      (c : byte; port : integer);
function  RcvPort       (port : integer) : byte;
function  PortDataRdy   (port : integer) : boolean;
function  NoCarrier     (port : integer) : boolean;

implementation

var
  Regs : Registers;

procedure GetPortStatus(port : integer);
begin
  Regs.DX := port;
  Regs.AH := 3;
  Intr($14, Regs);
  PortStat[port] := Regs.AX;
end; { GetPortStatus }

function BufferFull(port : integer) : boolean;
var result:word;
begin
  GetPortStatus(port);
  result := PortStat[port] and $F000;
  bufferfull := (result = $2000);
end;

function NoCarrier(port : integer) : boolean;
begin
  Regs.DX := port;
  Regs.AH := 3;
  Intr($14, Regs);
(*CarrierDetect := (Regs.AL AND $80) > 0;*)
  NoCarrier := ((Regs.AL and $80) = $00);
end;

{$F+}
procedure UnInitFossil;
(* InitFossil establishes this as the ExitProc, so that it
   will always be executed regardless of error condition. *)
begin
  ExitProc:=FossilExitSave;
  if FossilInstalled then begin
    repeat until ((not BufferFull(PortUsed)) or NoCarrier(PortUsed));
    Regs.DX := PortUsed;
    Regs.AH := 5;
    Intr($14, Regs);
    FossilInstalled := false;
  end;
end; { UnInitFossil }
{$F-}

procedure InitFossil(port : integer);
begin
  Regs.DX := port;
  Regs.BX := 0;
  Regs.CX := 0;
  Regs.AH := 4;
  Intr($14, Regs);
  FossilInstalled := (Regs.AX = $1954);
  FossilMaxFcn := Regs.BL;
  FossilVersion := Regs.BH;
  FossilExitSave := ExitProc;
  ExitProc := @UnInitFossil;
end; { InitFossil }

procedure XmitPort(c : byte; port : integer);
begin
  Regs.DX := port;
  Regs.AL := c;
  Regs.AH := 1;
  Intr($14, Regs);
  PortStat[port]:= Regs.AX;
end; { Xmit }

function RcvPort(port : integer) : byte;
begin
  Regs.DX := port;
  Regs.AH := 2;
  Intr($14, Regs);
  RcvPort := Regs.AL;
end; { RcvPort }

function PortDataRdy(port : integer) : boolean;
begin
  GetPortStatus(port);
  PortDataRdy := ((PortStat[port] and $100) <> 0);
end; { PortDataRdy }

begin
  { Initialization Code }
  FossilInstalled:= false;
end.
